package com.usky.system.controller.web;

import java.util.*;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.constants.CacheConstants;
import com.usky.common.core.utils.StringUtils;
import com.usky.common.redis.core.RedisService;
import com.usky.common.security.annotation.RequiresPermissions;
import com.usky.system.controller.web.page.TableDataInfo;
import com.usky.system.domain.SysUserOnline;
import com.usky.system.model.LoginUser;
import com.usky.system.service.ISysUserOnlineService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.slf4j.Logger;

/**
 * 在线用户监控
 *
 */
@RestController
@RequestMapping("/online")
public class SysUserOnlineController {
    private final Logger logger = LoggerFactory.getLogger(SysUserOnlineController.class);

    @Autowired
    private ISysUserOnlineService userOnlineService;

    @Autowired
    private RedisService redisService;

    private TableDataInfo getDataTable(List<SysUserOnline> userOnlineList, int pageNum, int pageSize, long total) {
        TableDataInfo dataInfo = new TableDataInfo();
        dataInfo.setRows(userOnlineList);
        dataInfo.setTotal(total);
        dataInfo.setPageNum(pageNum);
        dataInfo.setPageSize(pageSize);
        dataInfo.setPages((int) Math.ceil((double) total / pageSize));
        return dataInfo;
    }

    @GetMapping("/list")
    public ApiResult<TableDataInfo> list(@RequestParam(required = false, defaultValue = "1") Integer page,
                                         @RequestParam(required = false, defaultValue = "10") Integer size,
                                         @RequestParam(required = false) String ipaddr,
                                         @RequestParam(required = false) String userName) {
        try {
            Collection<String> keys = redisService.keys(CacheConstants.LOGIN_TOKEN_KEY + "*");
            if (keys == null || keys.isEmpty()) {
                logger.error("Redis keys is null or empty");
                return ApiResult.error("SYS-0000", "Redis keys is null or empty");
            }
            Map<String, List<SysUserOnline>> userOnlineMap = new HashMap<>();

            for (String key : keys) {
                // 检查键是否存在并且未过期
                if (redisService.hasKey(key)) {
                    LoginUser user = redisService.getCacheObject(key);
                    if (user != null) {
                        if (ipaddr != null && !ipaddr.equals(user.getIpaddr())) {
                            continue;
                        }
                        if (userName != null && !userName.equals(user.getUsername())) {
                            continue;
                        }
                        SysUserOnline online = userOnlineService.loginUserToUserOnline(user);
                        if (online != null) {
                            List<SysUserOnline> list = userOnlineMap.computeIfAbsent(user.getUsername(), k -> new ArrayList<>());
                            list.add(online);
                        }
                    }
                }
            }

            List<SysUserOnline> userOnlineList = new ArrayList<>();
            for (Map.Entry<String, List<SysUserOnline>> entry : userOnlineMap.entrySet()) {
                List<SysUserOnline> sortedList = entry.getValue();
                sortedList.sort(Comparator.comparing(SysUserOnline::getLoginTime).reversed());
                if (!sortedList.isEmpty()) {
                    SysUserOnline lastOnline = sortedList.get(0);
                    lastOnline.setLoginCount(sortedList.size()); // 设置登录次数
                    userOnlineList.add(lastOnline);
                }
            }

            // 按照登录时间降序排序
            userOnlineList.sort(Comparator.comparing(SysUserOnline::getLoginTime).reversed());

            long total = userOnlineList.size();
            int start = (page - 1) * size;
            int end = Math.min(start + size, (int) total);
            List<SysUserOnline> pageList = userOnlineList.subList(start, end);

            return ApiResult.success(getDataTable(pageList, page, size, total));
        } catch (Exception ex) {
            logger.error("Error processing /online/list", ex);
            return ApiResult.error("SYS-0001", "Internal system error", ex.getMessage());
        }
    }

    @GetMapping("/history")
    public ApiResult<TableDataInfo> listOtherLogins(@RequestParam String userName,
                                                    @RequestParam(required = false, defaultValue = "1") Integer page,
                                                    @RequestParam(required = false, defaultValue = "10") Integer size) {
        try {
            Collection<String> keys = redisService.keys(CacheConstants.LOGIN_TOKEN_KEY + "*");
            if (keys == null || keys.isEmpty()) {
                logger.error("Redis keys is null or empty");
                return ApiResult.error("SYS-0000", "Redis keys is null or empty");
            }
            List<SysUserOnline> userOnlineList = new ArrayList<>();

            for (String key : keys) {
                if (redisService.hasKey(key)) {
                    LoginUser user = redisService.getCacheObject(key);
                    if (user != null && user.getUsername().equals(userName)) {
                        SysUserOnline online = userOnlineService.loginUserToUserOnline(user);
                        if (online != null) {
                            userOnlineList.add(online);
                        }
                    }
                }
            }

            // 按照登录时间降序排序
            userOnlineList.sort(Comparator.comparing(SysUserOnline::getLoginTime).reversed());
            // 排除最后一条记录，只保留其他记录
//            if (!userOnlineList.isEmpty()) {
//                userOnlineList = userOnlineList.subList(1, userOnlineList.size());
//            }

            long total = userOnlineList.size();
            int start = (page - 1) * size;
            int end = Math.min(start + size, (int) total);
            List<SysUserOnline> pageList = userOnlineList.subList(start, end);

            return ApiResult.success(getDataTable(pageList, page, size, total));
        } catch (Exception ex) {
            logger.error("Error processing /online/listOtherLogins", ex);
            return ApiResult.error("SYS-0001", "Internal system error", ex.getMessage());
        }
    }

    /**
     * 强退用户
     */
//    @RequiresPermissions("monitor:online:forceLogout")
    @DeleteMapping("/{tokenId}")
    public ApiResult forceLogout(@PathVariable String tokenId) {
        try {
            if (StringUtils.isBlank(tokenId)) {
                logger.error("Token ID is blank for force logout");
                return ApiResult.error("SYS-0004", "Token ID is required for force logout");
            }

            String key = CacheConstants.LOGIN_TOKEN_KEY + tokenId;
            boolean deleted = redisService.deleteObject(key);
            if (deleted) {
                logger.info("User with tokenId {} has been forcibly logged out.", tokenId);
                return ApiResult.success();
            } else {
                logger.warn("Failed to find and delete user session with tokenId {}", tokenId);
                return ApiResult.error("SYS-0002", "Session not found or already deleted");
            }
        } catch (Exception ex) {
            logger.error("Error processing forceLogout for tokenId {}", tokenId, ex);
            return ApiResult.error("SYS-0003", "Internal system error during force logout", ex.getMessage());
        }
    }

    @DeleteMapping("/logoutMany")
    public ApiResult forceLogoutMany(@RequestBody List<String> tokenIds) {
        for (String tokenId : tokenIds) {
            try {
                if (StringUtils.isBlank(tokenId)) {
                    logger.error("Token ID is blank for force logout");
                    continue;
                }

                String key = CacheConstants.LOGIN_TOKEN_KEY + tokenId;
                boolean deleted = redisService.deleteObject(key);
                if (deleted) {
                    logger.info("User with tokenId {} has been forcibly logged out.", tokenId);
                } else {
                    logger.warn("Failed to find and delete user session with tokenId {}", tokenId);
                }
            } catch (Exception ex) {
                logger.error("Error processing forceLogout for tokenId {}", tokenId, ex);
                return ApiResult.error("SYS-0003", "Internal system error during force logout", ex.getMessage());
            }
        }
        return ApiResult.success("All selected tokens have been processed for force logout.");
    }
}